from Dictionary.Word import Word
from Language.TurkishLanguage import TurkishLanguage
from Corpus.Sentence import Sentence
from Corpus.SentenceSplitter import SentenceSplitter


class TurkishSplitter(SentenceSplitter):
    """
    The listContains method has a String array shortcuts which holds the possible abbreviations that might end with a
    '.' but not a sentence finisher word. It also takes a String as an input and loops through the shortcuts array and
    returns true if given String has any matching item in the shortcuts array.

    PARAMETERS
    ----------
    currentWord : str
        String input to check.

    RETURNS
    -------
    bool
        True if contains any abbreviations, False otherwise.
    """

    def listContains(self, currentWord: str) -> bool:
        shortcuts = {"alb", "bnb", "bkz", "bşk", "co", "dr", "dç", "der", "em", "gn",
                     "hz", "kd", "kur", "kuv", "ltd", "md", "mr", "mö", "muh", "müh",
                     "no", "öğr", "op", "opr", "org", "sf", "tuğ", "uzm", "vb", "vd",
                     "yön", "yrb", "yrd", "üniv", "fak", "prof", "dz", "yd", "krm", "gen",
                     "pte", "p", "av", "II", "III", "IV", "VI", "VII", "VIII", "IX",
                     "X", "XI", "XII", "XIII", "XIV", "XV", "XVI", "XVII", "XVIII", "XIX",
                     "XX", "tuğa", "plt", "tğm", "tic", "srv", "bl", "dipl", "not", "min",
                     "cul", "san", "rzv", "or", "kor", "tüm", "st", "sn", "fr", "pl",
                     "ka", "tk", "ko", "vs", "yard", "bknz", "doç", "gör", "müz", "oyn",
                     "m", "s", "kr", "ms", "hv", "uz", "re", "ph", "mc", "ed",
                     "km", "yb", "bk", "jr", "bn", "os", "mrs", "bld", "sen", "alm",
                     "sir", "ord", "dir", "yay", "man", "brm", "edt", "dec", "mah", "cad",
                     "vol", "kom", "sok", "apt", "elk", "mad", "ort", "cap", "ste", "exc",
                     "ef"}
        return currentWord in shortcuts

    """
    The isNextCharUpperCaseOrDigit method takes a String line and an int i as inputs. First it compares each char in
    the input line with " " and SEPARATORS ({@literal ()[]{}"'״＂՛}) and increment i by one until a mismatch or end of 
    line.

    When i equals to line length or contains one of the uppercase letters or digits it returns true, false otherwise.

    PARAMETERS
    ----------
    line : str
        String to check.
    i : int 
        defining starting index.
        
    RETURNS
    -------
    bool
        True if next char is uppercase or digit, False otherwise.
    """

    def isNextCharUpperCaseOrDigit(self, line: str, i: int) -> bool:
        while i < len(line) and (line[i] == ' ' or line[i] in SentenceSplitter.SEPARATORS):
            i = i + 1
        if i == len(line) or line[i] in TurkishLanguage.UPPERCASE_LETTERS + TurkishLanguage.DIGITS + "-":
            return True
        else:
            return False

    """
    The isPreviousWordUpperCase method takes a String line and an int i as inputs. First it compares each char in
    the input line with " " and checks each char whether they are lowercase letters or one of the qxw. And decrement
    input i by one till this condition is false.

    When i equals to -1 or contains one of the uppercase letters or one of the QXW it returns true, false otherwise.

    PARAMETERS
    ----------
    line : str
        String to check.
    i : int 
        defining ending index.
        
    RETURNS
    -------
    bool
        True if previous char is uppercase or one of the QXW, False otherwise.
    """

    def isPreviousWordUpperCase(self, line: str, i: int) -> bool:
        while i >= 0 and (line[i] == ' ' or line[i] in TurkishLanguage.LOWERCASE_LETTERS + "qxw"):
            i = i - 1
        if i == -1 or line[i] in TurkishLanguage.UPPERCASE_LETTERS + "QWX":
            return True
        else:
            return False

    """
    The isNextCharUpperCase method takes a String line and an int i as inputs. First it compares each char in
    the input line with " " and increment i by one until a mismatch or end of line.

    When i equals to line length or contains one of the uppercase letters it returns true, false otherwise.

    PARAMETERS
    ----------
    line : str
        String to check.
    i : int
        defining starting index.
        
    RETURNS
    -------
    bool
        True if next char is uppercase, False otherwise.
    """

    def isNextCharUpperCase(self, line: str, i: int) -> bool:
        while i < len(line) or line[i] == ' ':
            i = i + 1
        if i == len(line) or line[i] in TurkishLanguage.UPPERCASE_LETTERS + "\"\'":
            return True
        else:
            return False

    """
    The isNameShortcut method takes a String word as an input. First, if the word length is 1, and currentWord
    contains UPPERCASE_LETTERS letters than it returns true.

    Secondly, if the length of the word is 3 (i.e it is a shortcut) and it has a '.' at its 1st index and 
    currentWord's 2nd  index is an uppercase letter it also returns true. (Ex : m.A)

    PARAMETERS
    ----------
    currentWord : str
        String input to check whether it is a shortcut.
        
    RETURNS
    -------
    bool
        True if given input is a shortcut, False otherwise.
    """

    def isNameShortcut(self, currentWord: str) -> bool:
        if len(currentWord) == 1 and currentWord in TurkishLanguage.UPPERCASE_LETTERS:
            return True
        if len(currentWord) == 3 and currentWord[1] == '.' and currentWord[2] in TurkishLanguage.UPPERCASE_LETTERS:
            return True
        return False

    """
    The repeatControl method takes a String word as an input, and a boolean exceptionMode and compress the repetitive 
    chars. With the presence of exceptionMode it directly returns the given word. Then it declares a counter i and 
    loops till the end of the given word. It compares the char at index i with the char at index (i+2) if they are 
    equal then it compares the char at index i with the char at index (i+1) and increments i by one and returns 
    concatenated result String with char at index i.

    PARAMETERS
    ----------
    word : str
        String input.
    exceptionMode : bool
        boolean input for exceptional cases.
        
    RETURNS
    -------
    str
        String result.
    """

    def repeatControl(self, word: str, exceptionMode: bool) -> str:
        if exceptionMode:
            return word
        i = 0
        result = ""
        while i < len(word):
            if i < len(word) - 2 and word[i] == word[i + 1] and word[i] == word[i + 2]:
                while i < len(word) - 1 and word[i] == word[i + 1]:
                    i = i + 1
            result = result + word[i]
            i = i + 1
        return result

    """
    The isApostrophe method takes a String line and an integer i as inputs. Initially declares a String apostropheLetters
    which consists of abcçdefgğhıijklmnoöprsştuüvyzABCÇDEFGĞHIİJKLMNOÖPRSŞTUÜVYZ, âàáäãèéêëíîòóôûúqwxÂÈÉÊËÌÒÛQWX and 
    0123456789. Then, it returns true if the result of contains method which checks the existence of previous char and 
    next char at apostropheLetters returns true, returns false otherwise.

    PARAMETERS
    ----------
    line : str
        String input to check.
    i : int   
        index.
        
    RETURNS
    -------
    bool
        True if apostropheLetters contains previous char and next char, False otherwise.
    """

    def isApostrophe(self, line: str, i: int) -> bool:
        apostropheLetters = TurkishLanguage.LETTERS + TurkishLanguage.EXTENDED_LANGUAGE_CHARACTERS + TurkishLanguage.DIGITS
        if i + 1 < len(line):
            previousChar = line[i - 1]
            nextChar = line[i + 1]
            return previousChar in apostropheLetters and nextChar in apostropheLetters
        else:
            return False

    """
    The numberExistsBeforeAndAfter method takes a String line and an integer i as inputs. Then, it returns true if
    the result of contains method, which compares the previous char and next char with 0123456789, returns true and
    false otherwise.

    PARAMETERS
    ----------
    line : str
        String input to check.
    i : int   
        index.
        
    RETURNS
    -------
    bool
        True if previous char and next char is a digit, False otherwise.
    """

    def numberExistsBeforeAndAfter(self, line: str, i: int) -> bool:
        if i + 1 < len(line) and i > 0:
            previousChar = line[i - 1]
            nextChar = line[i + 1]
            return previousChar in TurkishLanguage.DIGITS and nextChar in TurkishLanguage.DIGITS
        else:
            return False

    """
    The isTime method takes a String line and an integer i as inputs. Then, it returns true if
    the result of the contains method, which compares the previous char, next char and two next chars with 0123456789,
    returns true and false otherwise.

    PARAMETERS
    ----------
    line : str
        String input to check.
    i : int   
        index.

    RETURNS
    -------
    bool
        True if previous char, next char and two next chars are digit, False otherwise.
    """

    def isTime(self, line: str, i: int) -> bool:
        if i + 2 < len(line):
            previousChar = line[i - 1]
            nextChar = line[i + 1]
            twoNextChar = line[i + 2]
            return previousChar in TurkishLanguage.DIGITS and nextChar in TurkishLanguage.DIGITS and twoNextChar in TurkishLanguage.DIGITS
        else:
            return False

    """
    The split method takes a String line as an input. Firstly it creates a new sentence as currentSentence a new list
    as sentences. Then loops till the end of the line and checks some conditions;
    If the char at ith index is a separator;

    ' : assigns currentWord as currentWord'
    { : increment the curlyBracketCount
    } : decrement the curlyBracketCount
    " : increment the specialQuotaCount
    " : decrement the specialQuotaCount
    ( : increment roundParenthesisCount
    ) : decrement roundParenthesisCount
    [ : increment bracketCount
    ] : decrement bracketCount
    " : assign quotaCount as 1- quotaCount
    ' : assign apostropheCount as 1- apostropheCount

    If the currentWord is not empty, it adds the currentWord after repeatControl to currentSentence.

    If the char at index i is " and  bracketCount, specialQuotaCount, curlyBracketCount, roundParenthesisCount, and
    quotaCount equal to 0 and also the next char is uppercase or digit, it adds currentSentence to sentences.

    If the char at ith index is a sentence ender;

    . and currentWord is www : assigns webMode as true. Ex: www.google.com
    . and currentWord is a digit or in web or e-mail modes : assigns currentWord as currentWord+char(i) Ex: 1.
    . and currentWord is a shortcut or an abbreviation : assigns currentWord as currentWord+char(i) and adds currentWord to currentSentence. Ex : bkz.
    ' and next char is uppercase or digit: add word to currentSentence as ' and add currentSentence to sentences.

    If the char at index i is ' ', i.e space, add word to currentSentence and assign "" to currentSentence.
    If the char at index i is -,  add word to currentSentence and add sentences when the wordCount of currentSentence greater than 0.

    If the char at ith index is a punctuation;
    : and if currentWord is "https" : assign webMode as true.
    , and there exists a number before and after : assign currentWord as currentWord+char(i) Ex: 1,2
    : and if line is a time : assign currentWord as currentWord+char(i) Ex: 12:14:24
    - and there exists a number before and after : assign currentWord as currentWord+char(i) Ex: 12-1
    {@literal @} : assign emailMode as true.

    PARAMETERS
    ----------
    line : str
        String input to split.
        
    RETURNS
    -------
    list
        sentences list which holds split line.
    """

    def split(self, line: str) -> list:
        emailMode = False
        webMode = False
        i = 0
        specialQuotaCount = 0
        roundParenthesisCount = 0
        bracketCount = 0
        curlyBracketCount = 0
        quotaCount = 0
        apostropheCount = 0
        currentSentence = Sentence()
        currentWord = ""
        sentences = []
        while i < len(line):
            if line[i] in SentenceSplitter.SEPARATORS:
                if line[i] == '\'' and currentWord != "" and self.isApostrophe(line, i):
                    currentWord = currentWord + line[i]
                else:
                    if currentWord != "":
                        currentSentence.addWord(Word(self.repeatControl(currentWord, webMode or emailMode)))
                    currentSentence.addWord(Word("" + line[i]))
                    currentWord = ""
                    if line[i] == '{':
                        curlyBracketCount = curlyBracketCount + 1
                    elif line[i] == '}':
                        curlyBracketCount = curlyBracketCount - 1
                    elif line[i] == '\uFF02':
                        specialQuotaCount = specialQuotaCount + 1
                    elif line[i] == '\u05F4':
                        specialQuotaCount = specialQuotaCount - 1
                    elif line[i] == '(':
                        roundParenthesisCount = roundParenthesisCount + 1
                    elif line[i] == ')':
                        roundParenthesisCount = roundParenthesisCount - 1
                    elif line[i] == '[':
                        bracketCount = bracketCount + 1
                    elif line[i] == ']':
                        bracketCount = bracketCount - 1
                    elif line[i] == '"':
                        quotaCount = 1 - quotaCount
                    elif line[i] == '\'':
                        apostropheCount = 1 - apostropheCount
                    if line[i] == '"' and bracketCount == 0 and specialQuotaCount == 0 and curlyBracketCount == 0 and \
                            roundParenthesisCount == 0 and quotaCount == 0 and self.isNextCharUpperCaseOrDigit(line,
                                                                                                               i + 1):
                        sentences.append(currentSentence)
                        currentSentence = Sentence()
            else:
                if line[i] in SentenceSplitter.SENTENCE_ENDERS:
                    if line[i] == '.' and currentWord == "www":
                        webMode = True
                    if line[i] == '.' and currentWord != "" and (
                            webMode or emailMode or line[i - 1] in TurkishLanguage.DIGITS):
                        currentWord = currentWord + line[i]
                    else:
                        if line[i] == '.' and (self.listContains(currentWord) or self.isNameShortcut(currentWord)):
                            currentWord = currentWord + line[i]
                            currentSentence.addWord(Word(currentWord))
                            currentWord = ""
                        else:
                            if currentWord != "":
                                currentSentence.addWord(Word(self.repeatControl(currentWord, webMode or emailMode)))
                            currentWord = "" + line[i]
                            i = i + 1
                            while i < len(line) and line[i] in SentenceSplitter.SENTENCE_ENDERS:
                                i = i + 1
                            i = i - 1
                            currentSentence.addWord(Word(currentWord))
                            if roundParenthesisCount == 0 and bracketCount == 0 and curlyBracketCount == 0 and \
                                    quotaCount == 0:
                                if i + 1 < len(line) and line[i + 1] == '\'' and apostropheCount == 1 and \
                                        self.isNextCharUpperCaseOrDigit(line, i + 2):
                                    currentSentence.addWord(Word("'"))
                                    i = i + 1
                                    sentences.append(currentSentence)
                                    currentSentence = Sentence()
                                else:
                                    if i + 2 < len(line) and line[i + 1] == ' ' and line[i + 2] == '\'' and \
                                            apostropheCount == 1 and self.isNextCharUpperCaseOrDigit(line, i + 3):
                                        currentSentence.addWord(Word("'"))
                                        i += 2
                                        sentences.append(currentSentence)
                                        currentSentence = Sentence()
                                    else:
                                        if self.isNextCharUpperCaseOrDigit(line, i + 1):
                                            sentences.append(currentSentence)
                                            currentSentence = Sentence()
                            currentWord = ""
                else:
                    if line[i] == ' ':
                        emailMode = False
                        webMode = False
                        if currentWord != "":
                            currentSentence.addWord(Word(self.repeatControl(currentWord, webMode or emailMode)))
                            currentWord = ""
                    else:
                        if line[i] == '-' and not webMode and roundParenthesisCount == 0 and \
                                self.isNextCharUpperCase(line, i + 1) and not self.isPreviousWordUpperCase(line, i - 1):
                            if currentWord != "" and currentWord not in TurkishLanguage.DIGITS:
                                currentSentence.addWord(Word(self.repeatControl(currentWord, webMode or emailMode)))
                            if currentSentence.wordCount() > 0:
                                sentences.append(currentSentence)
                            currentSentence = Sentence()
                            roundParenthesisCount = 0
                            bracketCount = 0
                            curlyBracketCount = 0
                            quotaCount = 0
                            specialQuotaCount = 0
                            if currentWord != "" and currentWord.matches("\\d+"):
                                currentSentence.addWord(Word(currentWord + " -"))
                            else:
                                currentSentence.addWord(Word("-"))
                            currentWord = ""
                        else:
                            if line[i] in SentenceSplitter.PUNCTUATION_CHARACTERS or \
                                    line[i] in TurkishLanguage.ARITHMETIC_CHARACTERS:
                                if line[i] == ':' and (currentWord == "http" or currentWord == "https"):
                                    webMode = True
                                if webMode:
                                    currentWord = currentWord + line[i]
                                else:
                                    if line[i] == ',' and self.numberExistsBeforeAndAfter(line, i):
                                        currentWord = currentWord + line[i]
                                    else:
                                        if line[i] == ':' and self.isTime(line, i):
                                            currentWord = currentWord + line[i]
                                        else:
                                            if line[i] == '-' and self.numberExistsBeforeAndAfter(line, i):
                                                currentWord = currentWord + line[i]
                                            else:
                                                if currentWord != "":
                                                    currentSentence.addWord(
                                                        Word(self.repeatControl(currentWord, webMode or emailMode)))
                                                currentSentence.addWord(Word("" + line[i]))
                                                currentWord = ""
                            else:
                                if line[i] == '@':
                                    currentWord = currentWord + line[i]
                                    emailMode = True
                                else:
                                    currentWord = currentWord + line[i]
            i = i + 1
        if currentWord != "":
            currentSentence.addWord(Word(self.repeatControl(currentWord, webMode or emailMode)))
        if currentSentence.wordCount() > 0:
            sentences.append(currentSentence)
        return sentences



from __future__ import annotations
from DataStructure.CounterHashMap import CounterHashMap
from Dictionary.Word import Word

from Corpus.LanguageChecker import LanguageChecker
from Corpus.Paragraph import Paragraph
from Corpus.Sentence import Sentence
from Corpus.SentenceSplitter import SentenceSplitter

import random

class Corpus:

    """
    Constructor of Corpus class which takes a file name as an input. Then reads the input file line by line
    and calls addSentence method with each read line.

    PARAMETERS
    ----------
    fileName : str
        String file name input that will be read.
    """
    def __init__(self, fileName = None):
        self.sentences = []
        self.paragraphs = []
        self.wordList = CounterHashMap()
        if fileName is not None:
            self.fileName = fileName
            file = open(fileName, "r")
            lines = file.readlines()
            for line in lines:
                self.addSentence(Sentence(line))

    """
    Another constructor of Corpus class which takes SentenceSplitter  as an input besides the file name.
    It reads input file line by line and calls the sentenceSplitter method with each line, then calls addSentence method
    with each sentence.

    PARAMETERS
    ----------
    fileName : str        
        String file name input that will be read.
    sentenceSplitter : SentenceSplitter
        SentenceSplitter type input.
    """
    def initWithSentenceSplitter(self, fileName: str, sentenceSplitter: SentenceSplitter):
        self.sentences = []
        self.paragraphs = []
        self.wordList = CounterHashMap()
        self.fileName = fileName
        file = open(fileName, "r")
        lines = file.readlines()
        for line in lines:
            sentences = sentenceSplitter.split(line)
            paragraph = Paragraph()
            for sentence in sentences:
                paragraph.addSentence(sentence)
            self.addParagraph(paragraph)

    """
    Another constructor of Corpus class which also takes languageChecker input besides file name input.
    It reads input file line by line and add each sentence also by using the languageChecker input which simply checks
    the validity of the sentence.

    PARAMETERS
    ----------
    fileName : str       
        String file name input that will be read.
    languageChecker : LanguageChecker
        LanguageChecker type input.
    """
    def initWithLanguageChecker(self, fileName: str, languageChecker: LanguageChecker):
        self.sentences = []
        self.paragraphs = []
        self.wordList = CounterHashMap()
        self.fileName = fileName
        if fileName is not None:
            file = open(fileName, "r")
            lines = file.readlines()
            for line in lines:
                sentence = Sentence()
                self.addSentence(sentence.initWithLanguageChecker(line, languageChecker))

    """
    The combine method takes a Corpus as an input and adds each sentence of sentences list.

    PARAMETERS
    ----------
    corpus : Corpus
        Corpus type input.
    """
    def combine(self, corpus: Corpus):
        for sentence in corpus.sentences:
            self.addSentence(sentence)

    """
    The addSentence method takes a Sentence as an input. It adds given input to sentences list and loops
    through the each word in sentence and puts these words into wordList CounterHashMap.

    PARAMETERS
    ----------
    s : Sentence
        Sentence type input that will be added to sentences list and its words will be added to wordList CounterHashMap.
    """
    def addSentence(self, s: Sentence):
        self.sentences.append(s)
        for i in range(s.wordCount()):
            w = s.getWord(i)
            self.wordList.put(w)

    """
    The numberOfWords method loops through the sentences list and accumulates the number of words in sentence.

    RETURNS
    -------
    int
        size which holds the total number of words.
    """
    def numberOfWords(self) -> int:
        size = 0
        for s in self.sentences:
            size += s.wordCount()
        return size

    """
    The contains method takes a String word as an input and checks whether wordList CounterHashMap has the
    given word and returns true if so, otherwise returns false.

    PARAMETERS
    ----------
    word : str
        String input to check.
        
    RETURNS
    -------
    bool
        True if wordList has the given word, False otherwise.
    """
    def contains(self, word: str) -> bool:
        return Word(word) in self.wordList

    """
    The addParagraph method takes a Paragraph type input. It gets the sentences in the given paragraph and
    add these to the sentences list and the words in the sentences to the wordList CounterHashMap.

    PARAMETERS
    ----------
    p : Paragraph
        Paragraph type input to add sentences and wordList.
    """
    def addParagraph(self, p: Paragraph):
        self.paragraphs.append(p)
        for i in range(p.sentenceCount()):
            self.addSentence(p.getSentence(i))

    """
    Getter for the file name.

    RETURNS
    -------
    str
        file name.
    """
    def getFileName(self) -> str:
        return self.fileName

    """
    Getter for the wordList.

    RETURNS
    -------
    set
        The keySet of wordList.
    """
    def getWordList(self) -> set:
        return self.wordList.keys()

    """
    The wordCount method returns the size of the wordList CounterHashMap.

    RETURNS
    -------
    int
        The size of the wordList CounterHashMap.
    """
    def wordCount(self) -> int:
        return len(self.wordList)

    """
    The getCount method returns the count value of given word.

    PARAMETERS
    ----------
    word : Word
        Word type input to check.
        
    RETURNS
    -------
    int
        The count value of given word.
    """
    def getCount(self, word: Word) -> int:
        return self.wordList[word]

    """
    The sentenceCount method returns the size of the sentences list.

    RETURNS
    -------
    int
        The size of the sentences list.
    """
    def sentenceCount(self) -> int:
        return len(self.sentences)

    """
    Getter for getting a sentence at given index.

    PARAMETERS
    ----------
    index : int
        index to get sentence from.
        
    RETURNS
    -------
    Sentence
        The sentence at given index.
    """
    def getSentence(self, index: int) -> Sentence:
        return self.sentences[index]

    """
    The paragraphCount method returns the size of the paragraphs list.

    RETURNS
    -------
    int
        The size of the paragraphs list.
    """
    def paragraphCount(self) -> int:
        return len(self.paragraphs)

    """
    Getter for getting a paragraph at given index.

    PARAMETERS
    ----------
    index : int
        index to get paragraph from.
        
    RETURNS
    -------
    Paragraph
        The paragraph at given index.
    """
    def getParagraph(self, index: int) -> Paragraph:
        return self.paragraphs[index]

    """
    The maxSentenceLength method finds the sentence with the maximum number of words and returns this number.

    RETURNS
    -------
    int
        maximum length.
    """
    def maxSentenceLength(self) -> int:
        maxLength = 0
        for s in self.sentences:
            if s.wordCount() + 1 > maxLength:
                maxLength = s.wordCount() + 1
        return maxLength

    """
    The getAllWordsAsList method creates new list of lists and adds each word in each sentence of sentences
    list into new list.

    RETURNS
    -------
    list
        Newly created and populated list.
    """
    def getAllWordsAsList(self) -> list:
        allWords = []
        for i in range(self.sentenceCount()):
            allWords.append(self.getSentence(i).getWords())
        return allWords

    """
    The shuffleSentences method randomly shuffles sentences list with given seed value.

    PARAMETERS
    ----------
    seed : int
        value to randomize shuffling.
    """
    def shuffleSentences(self, seed: int):
        random.shuffle(self.sentences, seed)

    """
    The getTrainCorpus method takes two integer inputs foldNo and foldCount for determining train data size and count 
    of fold respectively. Initially creates a new empty Corpus, then finds the sentenceCount as N. Then, starting from 
    the index 0 it loops through the index (foldNo * N) / foldCount and add each sentence of sentences list to new 
    Corpus. Later on, starting from the index ((foldNo + 1) * N) / foldCount, it loops through the index N and add each 
    sentence of sentences list to new Corpus.

    PARAMETERS
    ----------
    foldNo : int   
        Integer input for train set size.
    foldCount : int
        Integer input for counting fold.
        
    RETURNS
    -------
    Corpus
        The newly created and populated Corpus.
    """
    def getTrainCorpus(self, foldNo: int, foldCount: int) -> Corpus:
        trainCorpus = Corpus()
        N = self.sentenceCount()
        for i in range((foldNo * N) // foldCount):
            trainCorpus.addSentence(self.sentences[i])
        for i in range(((foldNo + 1) * N) // foldCount, N):
            trainCorpus.addSentence(self.sentences[i])
        return trainCorpus

    """
    The getTestCorpus method takes two integer inputs foldNo and foldCount for determining test data size and count of
    fold respectively. Initially creates a new empty Corpus, then finds the sentenceCount as N. Then, starting from the 
    index (foldNo * N) / foldCount it loops through the index ((foldNo + 1) * N) / foldCount and add each sentence of 
    sentences list to new Corpus.

    PARAMETERS
    ----------
    foldNo : int   
        Integer input for test size.
    foldCount : int
        Integer input counting fold.
        
    RETURNS
    -------
    Corpus
        The newly created and populated Corpus.
    """
    def getTestCorpus(self, foldNo: int, foldCount: int) -> Corpus:
        testCorpus = Corpus()
        N = self.sentenceCount()
        for i in range((foldNo * N) // foldCount, ((foldNo + 1) * N) // foldCount):
            testCorpus.addSentence(self.sentences[i])
        return testCorpus



from __future__ import annotations
from Dictionary.Word import Word

from Corpus.LanguageChecker import LanguageChecker


class Sentence:

    """
    Another constructor of Sentence class which takes a fileName as an input. It reads each word in the file
    and adds to words list.

    PARAMETERS
    ----------
    fileName: str
        input file to read words from.
    """
    def __init__(self, fileName = None):
        self.words = []
        if fileName is not None:
            inputFile = open(fileName, "r")
            lines = inputFile.readlines()
            for line in lines:
                wordList = line.split(" ")
                for word in wordList:
                    self.words.append(Word(word))
            inputFile.close()

    """
    Another constructor of Sentence class which takes a sentence String as an input. It parses the sentence by
    " " and adds each word to the newly created words list.

    PARAMETERS
    ----------
    sentence : str
        String input to parse.
    """
    def initWithSentence(self, sentence: str):
        wordArray = sentence.split(" ")
        for word in wordArray:
            if len(word) > 0:
                self.words.append(Word(word))

    """
    Another constructor of Sentence class with two inputs; a String sentence and a {@link LanguageChecker}
     * languageChecker. It parses a sentence by " " and then check the language considerations. If it is a valid word,
     * it adds this word to the newly created {@link ArrayList} words.
     *
     * @param sentence        String input.
     * @param languageChecker {@link LanguageChecker} type input.
    """
    def initWithLanguageChecker(self, sentence: str, languageChecker: LanguageChecker):
        wordArray = sentence.split(" ")
        for word in wordArray:
            if len(word) > 0 and languageChecker.isValidWord(word):
                self.words.append(Word(word))

    """
    The equals method takes a Sentence as an input. First compares the sizes of both words lists and words
    of the Sentence input. If they are not equal then it returns false. Than it compares each word in the list.
    If they are equal, it returns true.

    PARAMETERS
    ----------
    s : Sentence
        Sentence to compare.
        
    RETURNS
    -------
    bool
        True if words of two sentences are equal.
    """
    def __eq__(self, s: Sentence) -> bool:
        if len(self.words) != len(s.words):
            return False
        for i in range(len(self.words)):
            if self.words[i].getName() != s.words[i].getName():
                return False
        return True

    """
    The getWord method takes an index input and gets the word at that index.

    PARAMETERS
    ----------
    index : int
        is used to get the word.

    RETURNS
    -------
    Word
        the word in given index.
    """
    def getWord(self, index: int) -> Word:
        return self.words[index]

    """
    The getWords method returns the words list.

    RETURNS
    -------
    list
        Words ArrayList.
    """
    def getWords(self) -> list:
        return self.words

    """
    The getStrings method loops through the words list and adds each words' names to the newly created result list.

    RETURNS
    -------
    list
        Result list which holds names of the words.
    """
    def getStrings(self) -> list:
        result = []
        for word in self.words:
            result.append(word.getName())
        return result

    """
    The getIndex method takes a word as an input and finds the index of that word in the words list if it exists.

    PARAMETERS
    ----------
    word : Word
        Word type input to search for.
        
    RETURNS
    -------
    int
        Index of the found input, -1 if not found.
    """
    def getIndex(self, word: Word) -> int:
        return self.words.index(word)

    """
    The wordCount method finds the size of the words list.

    RETURNS
    -------
    int
        The size of the words list.
    """
    def wordCount(self) -> int:
        return len(self.words)

    """
    The addWord method takes a word as an input and adds this word to the words list.
    
    PARAMETERS
    ----------
    word : Word
        Word to add words list.
    """
    def addWord(self, word: Word):
        self.words.append(word)

    """
    The charCount method finds the total number of chars in each word of words list.

    RETURNS
    -------
    int
        number of the chars in the whole sentence.
    """
    def charCount(self) -> int:
        total = 0
        for word in self.words:
            total += word.charCount()
        return total

    """
    The replaceWord method takes an index and a word as inputs. It removes the word at given index from words
    list and then adds the given word to given index of words.

    PARAMETERS
    ----------
    i : int      
        index.
    newWord : Word
        to add the words list.
    """
    def replaceWord(self, i: int, newWord: Word):
        self.words.pop(i)
        self.words.insert(i, newWord)

    """
    The safeIndex method takes an index as an input and checks whether this index is between 0 and the size of the 
    words.

    PARAMETERS
    ----------
    index : int
        is used to check the safety.
        
    RETURNS
    -------
    bool
        true if an index is safe, false otherwise.
    """
    def safeIndex(self, index: int) -> bool:
        return 0 <= index < len(self.words)

    """
    The overridden toString method returns an accumulated string of each word in words list.

    RETURNS
    -------
    str
        String result which has all the word in words list.
    """
    def __str__(self) -> str:
        if len(self.words) > 0:
            result = self.words[0].__str__()
            for i in range(1, len(self.words)):
                result = result + " " + self.words[i].__str__()
            return result
        else:
            return ""

    """
    The toWords method returns an accumulated string of each word's names in words list.

    RETURNS
    -------
    str
        String result which has all the names of each item in words list.
    """
    def toString(self) -> str:
        if len(self.words) > 0:
            result = self.words[0].getName()
            for i in range(1, len(self.words)):
                result = result + " " + self.words[i].getName()
            return result
        else:
            return ""



from setuptools import setup

setup(
    name='NlpToolkit-Corpus',
    version='1.0.0',
    packages=['Corpus'],
    url='https://github.com/olcaytaner/Corpus-Py',
    license='',
    author='olcaytaner',
    author_email='olcaytaner@isikun.edu.tr',
    description='Corpus library'
)



from Language.TurkishLanguage import TurkishLanguage

from Corpus.LanguageChecker import LanguageChecker
from Corpus.SentenceSplitter import SentenceSplitter


class TurkishChecker(LanguageChecker):

    """
    The isValidWord method takes an input String as a word than define all valid characters as a validCharacters String which has
    letters (abcçdefgğhıijklmnoöprsştuüvyzABCÇDEFGĞHIİJKLMNOÖPRSŞTUÜVYZ),
    extended language characters (âàáäãèéêëíîòóôûúqwxÂÈÉÊËÌÒÛQWX),
    digits (0123456789),
    separators ({@literal ()[]{}"'״＂՛}),
    sentence enders (.?!…),
    arithmetic chars (+-/=\*),
    punctuation chars (,:;),
    special-meaning chars

    Then, loops through input word's each char and if a char in word does not in the validCharacters string it returns
    false, true otherwise.

    PARAMETERS
    ----------
    word : str
        String to check validity.

    RETURNS
    -------
    bool
        True if each char in word is valid, False otherwise.
    """
    def isValidWord(self, word: str) -> bool:
        specialMeaningCharacters = "$\\_|@%#£§&><"
        validCharacters = TurkishLanguage.LETTERS + TurkishLanguage.EXTENDED_LANGUAGE_CHARACTERS + TurkishLanguage.DIGITS + SentenceSplitter.SEPARATORS + SentenceSplitter.SENTENCE_ENDERS + TurkishLanguage.ARITHMETIC_CHARACTERS + SentenceSplitter.PUNCTUATION_CHARACTERS + specialMeaningCharacters
        for i in range(len(word)):
            if word[i] not in validCharacters:
                return False
        return True
